@@ -5,11 +5,14 @@ class Derived(Foo) :
         Foo.__init__(self)
 
     def do_foo(self):
-        self.val -= 1
+        self.val = self.val - 1
 
 
 d = Derived()
 d.run()
 
 if d.val >= 0:
+    print d.val
     raise RuntimeError
+
+d.stop()
