@@ -11,10 +11,17 @@
  * clisp language module for SWIG.
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_clisp_cxx[] = "$Id: clisp.cxx 12428 2011-02-04 19:43:24Z wsfulton $";
-
 #include "swigmod.h"
 
+static const char *usage = (char *) "\
+CLISP Options (available with -clisp)\n\
+     -extern-all       - Create clisp definitions for all the functions and\n\
+                         global variables otherwise only definitions for\n\
+                         externed functions and variables are created.\n\
+     -generate-typedef - Use def-c-type to generate shortcuts according to the\n\
+                         typedefs in the input.\n\
+";
+
 class CLISP:public Language {
 public:
   File *f_cl;
@@ -48,15 +55,7 @@ void CLISP::main(int argc, char *argv[]) {
 
   for (i = 1; i < argc; i++) {
     if (!strcmp(argv[i], "-help")) {
-      Printf(stdout, "clisp Options (available with -clisp)\n");
-      Printf(stdout,
-	     " -extern-all\n"
-	     "\t If this option is given then clisp definitions for all the functions\n"
-	     "and global variables will be created otherwise only definitions for \n"
-	     "externed functions and variables are created.\n"
-	     " -generate-typedef\n"
-	     "\t If this option is given then def-c-type will be used to generate shortcuts\n"
-	     "according to the typedefs in the input.\n");
+      Printf(stdout, "%s\n", usage);
     } else if ((Strcmp(argv[i], "-extern-all") == 0)) {
       extern_all_flag = 1;
       Swig_mark_arg(i);
@@ -77,13 +76,13 @@ int CLISP::top(Node *n) {
   /* Get the output file name */
   String *outfile = Getattr(n, "outfile");
 
-  if (!outfile)
-    output_filename = outfile;
-  else {
-    output_filename = NewString("");
-    Printf(output_filename, "%s%s.lisp", SWIG_output_directory(), module);
+  if (!outfile) {
+    Printf(stderr, "Unable to determine outfile\n");
+    SWIG_exit(EXIT_FAILURE);
   }
 
+  output_filename = NewStringf("%s%s.lisp", SWIG_output_directory(), module);
+
   f_cl = NewFile(output_filename, "w+", SWIG_output_files());
   if (!f_cl) {
     FileErrorDisplay(output_filename);
@@ -131,17 +130,16 @@ int CLISP::top(Node *n) {
 
   for (len--; len >= 0; len--) {
     end--;
-    Seek(f_cl, len, SEEK_SET);
+    (void)Seek(f_cl, len, SEEK_SET);
     int ch = Getc(f_cl);
-    Seek(f_cl, end, SEEK_SET);
+    (void)Seek(f_cl, end, SEEK_SET);
     Putc(ch, f_cl);
   }
 
   Seek(f_cl, 0, SEEK_SET);
   Write(f_cl, Char(header), Len(header));
 
-  Close(f_cl);
-  Delete(f_cl);			// Deletes the handle, not the file
+  Delete(f_cl);
 
   return SWIG_OK;
 }
