@@ -1,30 +1,29 @@
 /* ----------------------------------------------------------------------------- 
- * See the LICENSE file for information on copyright, usage and redistribution
- * of SWIG, and the README file for authors - http://www.swig.org/release.html.
+ * This file is part of SWIG, which is licensed as a whole under version 3 
+ * (or any later version) of the GNU General Public License. Some additional
+ * terms also apply to certain portions of SWIG. The full details of the SWIG
+ * license and copyrights can be found in the LICENSE and COPYRIGHT files
+ * included with the SWIG source code as distributed by the SWIG developers
+ * and at http://www.swig.org/legal.html.
  *
  * module.cxx
  *
  * This file is responsible for the module system.  
  * ----------------------------------------------------------------------------- */
 
-char cvsroot_module_cxx[] = "$Header: /cvsroot/swig/SWIG/Source/Modules/module.cxx,v 1.3 2006/03/06 22:50:57 wsfulton Exp $";
-
 #include "swigmod.h"
 
 struct Module {
-  ModuleFactory   fac;
-  char            *name;
-  Module          *next;
-  Module(const char *n, ModuleFactory f) {
+  ModuleFactory fac;
+  char *name;
+  Module *next;
+   Module(const char *n, ModuleFactory f) {
     fac = f;
-    name = new char[strlen(n)+1];
-    strcpy(name, n);
-    next = 0;
-  }
-
-  ~Module()
-  {
-    delete[] name;
+    name = new char[strlen(n) + 1];
+     strcpy(name, n);
+     next = 0;
+  } ~Module() {
+    delete[]name;
   }
 };
 
@@ -37,7 +36,7 @@ static Module *modules = 0;
  * ----------------------------------------------------------------------------- */
 
 void Swig_register_module(const char *n, ModuleFactory f) {
-  Module *m = new Module(n,f);
+  Module *m = new Module(n, f);
   m->next = modules;
   modules = m;
 }
@@ -51,7 +50,7 @@ void Swig_register_module(const char *n, ModuleFactory f) {
 ModuleFactory Swig_find_module(const char *name) {
   Module *m = modules;
   while (m) {
-    if (strcmp(m->name,name) == 0) {
+    if (strcmp(m->name, name) == 0) {
       return m->fac;
     }
     m = m->next;
